from abc import ABC
from typing import Any, Optional

from llama_index.core.embeddings.base import BaseEmbedding
from unstract.adapters.base import Adapter
from unstract.adapters.enums import AdapterTypes


class EmbeddingAdapter(Adapter, ABC):
    def __init__(self, name: str):
        super().__init__(name)
        self.name = name

    @staticmethod
    def get_id() -> str:
        return ""

    @staticmethod
    def get_name() -> str:
        return ""

    @staticmethod
    def get_description() -> str:
        return ""

    @staticmethod
    def get_icon() -> str:
        return ""

    @staticmethod
    def get_json_schema() -> str:
        return ""

    @staticmethod
    def get_adapter_type() -> AdapterTypes:
        return AdapterTypes.EMBEDDING

    def get_embedding_instance(
        self, embed_config: dict[str, Any]
    ) -> Optional[BaseEmbedding]:
        return None



from typing import Any, Optional


class AdapterBaseException(Exception):
    """Base class for exceptions from Unstract adapters."""

    def __init__(
        self, *args: Any, user_message: Optional[str] = None, **kwargs: Any
    ) -> None:
        super().__init__(*args, **kwargs)
        self._user_message = user_message

    @property
    def user_message(self) -> Optional[str]:
        return self._user_message


class AdapterError(AdapterBaseException):
    """Exceptions related to adapters."""

    def __init__(
        self,
        message: str,
        *args: Any,
        treat_as_user_message: bool = False,
        **kwargs: Any,
    ) -> None:
        user_message = message if treat_as_user_message else None
        super().__init__(*args, user_message=user_message, **kwargs)
        self.message = message

    def __str__(self) -> str:
        return f"{self.message}"



import os
from typing import Any, Optional

from llama_index.llms import OpenAI
from llama_index.llms.llm import LLM
from unstract.adapters.exceptions import AdapterError
from unstract.adapters.llm.constants import LLMKeys
from unstract.adapters.llm.helper import LLMHelper
from unstract.adapters.llm.llm_adapter import LLMAdapter


class Constants:
    MODEL = "model"
    API_KEY = "api_key"
    MAX_RETIRES = "max_retries"
    ADAPTER_NAME = "adapter_name"
    TIMEOUT = "timeout"
    API_BASE = "api_base"
    API_VERSION = "api_version"


class OpenAILLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("OpenAI")
        self.config = settings

    @staticmethod
    def get_id() -> str:
        return "openai|502ecf49-e47c-445c-9907-6d4b90c5cd17"

    @staticmethod
    def get_name() -> str:
        return "OpenAI"

    @staticmethod
    def get_description() -> str:
        return "OpenAI LLM"

    @staticmethod
    def get_icon() -> str:
        return (
            "https://storage.googleapis.com/pandora-static/"
            "adapter-icons/OpenAI.png"
        )

    @staticmethod
    def get_json_schema() -> str:
        f = open(f"{os.path.dirname(__file__)}/static/json_schema.json")
        schema = f.read()
        f.close()
        return schema

    def get_llm_instance(self) -> Optional[LLM]:
        try:
            timeout = float(
                self.config.get(Constants.TIMEOUT, LLMKeys.DEFAULT_TIMEOUT)
            )
            if self.config.get(Constants.MAX_RETIRES) is not None:
                llm = OpenAI(
                    model=str(self.config.get(Constants.MODEL)),
                    api_key=str(self.config.get(Constants.API_KEY)),
                    api_base=str(self.config.get(Constants.API_BASE)),
                    api_version=str(self.config.get(Constants.API_VERSION)),
                    max_retries=int(self.config.get(Constants.MAX_RETIRES, 3)),
                    api_type="openai",
                    temperature=0,
                    timeout=timeout,
                )
            else:
                llm = OpenAI(
                    model=str(self.config.get(Constants.MODEL)),
                    api_key=str(self.config.get(Constants.API_KEY)),
                    api_base=str(self.config.get(Constants.API_BASE)),
                    api_version=str(self.config.get(Constants.API_VERSION)),
                    api_type="openai",
                    temperature=0,
                    timeout=timeout,
                )
            return llm
        except Exception as e:
            raise AdapterError(str(e))

    def test_connection(self) -> bool:
        llm = self.get_llm_instance()
        test_result: bool = LLMHelper.test_llm_instance(llm=llm)
        return test_result



import logging
import os
from typing import Any, Optional

import requests
from requests import Response

from unstract.adapters.exceptions import AdapterError
from unstract.adapters.x2text.constants import X2TextConstants
from unstract.adapters.x2text.llm_whisperer.src.constants import (
    ExtractionModes,
    OCRModes,
)
from unstract.adapters.x2text.x2text_adapter import X2TextAdapter

logger = logging.getLogger(__name__)


class Constants:
    URL = "url"
    TEST_CONNECTION = "test-connection"
    PROCESS = "process"
    EXTRACTION_MODE = "extraction_mode"
    OCR_MODE = "ocr_mode"


class LLMWhisperer(X2TextAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("LLMWhisperer")
        self.config = settings

    @staticmethod
    def get_id() -> str:
        return "llmwhisperer|0a1647f0-f65f-410d-843b-3d979c78350e"

    @staticmethod
    def get_name() -> str:
        return "LLMWhisperer"

    @staticmethod
    def get_description() -> str:
        return "LLMWhisperer X2Text"

    @staticmethod
    def get_icon() -> str:
        return (
            "https://storage.googleapis.com/pandora-static/"
            "adapter-icons/LLMWhisperer.png"
        )

    @staticmethod
    def get_json_schema() -> str:
        f = open(f"{os.path.dirname(__file__)}/static/json_schema.json")
        schema = f.read()
        f.close()
        return schema

    def _make_request(
        self, request_type: str, **kwargs: Optional[dict[Any, Any]]
    ) -> Response:
        llm_whisperer_svc_url = (
            f"{self.config.get(Constants.URL)}"
            f"/api/v1/llm-whisperer/{request_type}"
        )
        platform_service_api_key = self.config.get(
            X2TextConstants.PLATFORM_SERVICE_API_KEY
        )

        headers = {
            "accept": "application/json",
            "Authorization": f"Bearer {platform_service_api_key}",
        }

        # Add files only if the request is for process
        files = None
        if "files" in kwargs:
            files = kwargs["files"] if kwargs["files"] is not None else None

        body = {
            Constants.EXTRACTION_MODE: self.config.get(
                Constants.EXTRACTION_MODE, ExtractionModes.TEXT.value
            ),
            Constants.OCR_MODE: self.config.get(
                Constants.OCR_MODE, OCRModes.LINE_PRINTER.value
            ),
        }

        response = requests.post(
            llm_whisperer_svc_url, headers=headers, files=files, data=body
        )
        return response

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        **kwargs: dict[Any, Any],
    ) -> str:
        try:
            input_f = open(input_file_path, "rb")
            files = {"file": input_f}
            response = self._make_request(
                request_type=Constants.PROCESS, files=files, **kwargs
            )
            if response.status_code != 200:
                logger.error(
                    "Error in LLM Whisperer process document: "
                    f"[{response.status_code}] {response.reason}"
                )
                raise AdapterError(
                    f"{response.status_code} - {response.reason}"
                )
            else:
                if response.content is not None:
                    if isinstance(response.content, bytes):
                        output = response.content.decode("utf-8")
                    if output_file_path is not None:
                        with open(output_file_path, "w", encoding="utf-8") as f:
                            f.write(output)
                            f.close()
                    return output
        except Exception as e:
            logger.error(f"Error occured while processing document {e}")
            if not isinstance(e, AdapterError):
                raise AdapterError(str(e))
            else:
                raise e
        finally:
            if input_f is not None:
                input_f.close()

    def test_connection(self) -> bool:
        try:
            response = self._make_request(Constants.TEST_CONNECTION)
            if response.status_code != 200:
                logger.error(
                    "Error in LLM Whisperer test-connection: "
                    f"[{response.status_code}] {response.reason}"
                )

                raise AdapterError(
                    f"{response.status_code} - {response.reason}"
                )
            else:
                return True
        except Exception as e:
            logger.error(f"Error occured while testing adapter {e}")
            if not isinstance(e, AdapterError):
                raise AdapterError(str(e))
            else:
                raise e
